#!/bin/sh

################################################################
# Datapath Ltd. Vision Linux Driver
# install.sh
# Date: 03/10/2013 
# support@datapath.co.uk
################################################################

SILENT="NO"
USER_MODE="NO"
VERSION="NO"
NOSHUTDOWN="NO"

INSTALL_RGB133_CONF="NO"

KERNELVERSION=`uname -r`
KVERSION=`echo $KERNELVERSION | cut -d . -f 1`
KPATCHLEVEL=`echo $KERNELVERSION | cut -d . -f 2`
KSUBLEVEL=`echo $KERNELVERSION | cut -d . -f 3`

ARCH=`uname -m | sed -e 's/i.86/i386/'`
if [ "$ARCH" = "i386" ] ; then
   ARCH_BITS=32
   NOT_ARCH_BITS=64
else
   ARCH_BITS=64
   NOT_ARCH_BITS=32
fi

RT=""
if [ "$ARCH_BITS" = "64" ] ; then
  KVER=`uname -r`
  PREEMPT_RT=`cat /boot/config-$KVER | grep "PREEMPT_RT"`
  if [ -n "$PREEMPT_RT" ] ; then
     RT="RT."
  fi
fi

print_short_version ( )
{
   VERSION="7.14.0.236"
   echo -n "$VERSION"
   echo -n "$VERSION" >> install.log
}

print_header ( )
{
   echo
   echo >> install.log
   echo -n "Datapath Ltd. Vision Linux Driver (v"
   echo -n "Datapath Ltd. Vision Linux Driver (v" >> install.log
   print_short_version
   echo ")"
   echo ")" >> install.log
   echo
   echo >> install.log
}

usage ( )
{
   print_header
   echo "Usage: $0 [-u] [-s]"
   echo
   echo "-u: Install the user mode driver."
   echo "    Default mode is to install kernel mode"
   echo "    driver when flag is not present."
   if [ "$1" = "USE_LICENCE" ] ; then
      echo "-s: Silently install, automatically accept"
      echo "    the Datapath Ltd. Software License."
   else
      echo "-s: Silently install."
   fi
   exit
}

while getopts "suvxn" flag ; do
   case "$flag" in
      s)SILENT="YES";;
      u)USER_MODE="YES";;
      v)VERSION="YES";;
      n)NOSHUTDOWN="YES";;
      x)usage;;
      *)usage;;
   esac
done

print_long_version ( )
{
   VERSION="7.14.0.236"
   echo
   echo "Datapath Ltd. Vision Linux Driver"
   echo "Version: $VERSION"
   echo 
}

print_license ( )
{
   echo
   echo "By installing this driver package you agree to be bound to the"
   echo "Datapath Ltd. Software Licence.  Full terms and conditions can be"
   echo "read in the file: `pwd`/docs/LICENCE"
   echo
   echo "If this is not acceptable please exit now and remove this"
   echo "driver installation package (`pwd`)."
   echo

   if [ "$SILENT" = "YES" ] ; then
      echo "Datapath Ltd. Software License has been automatically accepted."
      echo
      echo "[Press CTRL+C to cancel installation]"
      echo -n "Installation will commence in 10 seconds..."
      echo -n "\033[13D"
      COUNTDOWN="10"
      while [ $COUNTDOWN -gt 0 ] ; do
        sleep 1
        COUNTDOWN=$(($COUNTDOWN-1))
        echo -n " $COUNTDOWN"
        echo -n "\033[2D"
      done
      echo
      echo
   else
      echo "Do you wish to accept the Datapath Ltd. Software License and"
      echo -n "continue with the installation (Y/N) [N]: "
      read cont

      if [ "$cont" != "YES" -a "$cont" != "yes" -a \
         "$cont" != "Y" -a "$cont" != "y" ] ; then
         echo
         echo "Cancelling installation, please remove this driver installation"
         echo "package from [`pwd`]."
         exit
      fi
   fi

   echo "Commencing installation..."
   echo
}

print_help ( )
{
   echo
   echo "====================================================================="
   echo
   echo "If you are experiencing difficulty with this installation"
   echo "please contact support@datapath.co.uk."
   echo
   echo "Please include details of computer system such as motherboard"
   echo "type, processor type, RAM and the number of (and type of)"
   echo "Datapath Ltd. cards including positions in the system."
   echo
   echo "Please include details of Linux kernel version, and if"
   echo "appropriate, OS Distribution (Ubuntu, Fedora Core,"
   echo "Red Hat Enterprise etc)."
   echo
   echo "Please also run the provided diagnostics harvesting script"
   echo "(for more details please see the DIAG file included with this"
   echo "release)."
   echo
   echo " # cd <install_dir>"
   echo " # ./scripts/diag.sh"
   echo
   echo "====================================================================="
   echo
}

print_qt_help ( )
{
   echo
   echo "====================================================================="
   echo
   echo "The Vision sample application has not been installed because"
   echo "the required Qt4 libraries are not installed in this system."
   echo "At the very least you will need libQtCore and libQtGui."
   echo
   echo "For more information please contact support@datapath.co.uk."
   echo
   echo "Note: You may just need to add the location of your Qt libraries to"
   echo "      the installation users LD_LIBRARY_PATH environment variable."
   echo
   echo "====================================================================="
   echo
}

print_v4l2_help ( )
{
   echo
   echo "====================================================================="
   echo
   echo "The Vision sample application has not been installed because"
   echo "the required v4l2 libraries are not installed in this system."
   echo "At the very least you will need libv4l2.so and libv4lconvert.so."
   echo
   echo "For more information please contact support@datapath.co.uk."
   echo
   echo "====================================================================="
   echo
}

print_shutdown ( )
{
   echo
   echo "====================================================================="
   echo "========================= SHUTDOWN REQUIRED ========================="
   echo "====================================================================="
   echo "  In order to use the upgraded firmware this"
   echo "  machine must be fully powered off."
   echo

   if [ "$NOSHUTDOWN" != "YES" ] ; then
   echo -n "  Shutdown now? [Y/N] (default Y): "
   read option
   if [ "$option" = "y" -o "$option" = "Y" -o "$option" = "" ] ; then
      if [ -e /sbin/poweroff ] ; then
         /sbin/poweroff
      else
         echo "Unable to automatically poweroff, please shutdown manually."
      fi
   fi
   fi
   echo "====================================================================="
}

print_missing_tools ( )
{
   echo
   echo "Your system has one or more system tools missing which are"
   echo "required to compile and load the Vision Linux driver."
   echo
   echo "Required packages: make, gcc, binutils"
   print_help
   exit
}

print_missing_kernel_devel ( )
{
   echo
   echo "Your system is missing one or more kernel development packages which"
   echo "is required to build and load the Vision Linux driver."
   echo
   echo "Required packages: kernel-source, kernel-devel"
   echo
   echo "Please make sure that the correct versions of these packages are"
   echo "installed.  Versions required: `uname -r`"
   print_help
   exit
}

start_progress ( )
{
   touch .$1_lock > /dev/null 2>&1
   ./scripts/progress.sh .$1_lock &
}

stop_progress ( )
{
   rm -f .$1_lock > /dev/null 2>&1
}

sanity_check ( )
{
   # Check important tools
   IMPORTANT_TOOLS="make gcc ld"
   for tool in $IMPORTNANT_TOOLS ; do
      $tool --help > /dev/null 2>&1
      RET=$?
      if [ $RET -ne 0 ] ; then
         print_missing_tools
      fi
   done

   # Check importnant packages
   IMPORTANT_PACKAGE_FILES="scripts/mod/modpost"
   IMPORTANT_PACKAGE_DIRS="include/linux/media"
   KERNERL_BASE="/lib/modules/`uname -r`"
   KERNEL_BUILD="$KERNEL_BASE/build"
   KERNEL_SOURCE="$KERNEL_BASE/source"
   
   IMPORTANT_FILES_FOUND="NO"
   for file in $IMPORTANT_PACKAGE_FILES ; do
      if [ -f $KERNEL_BUILD/$file ] ; then
         IMPORTANT_FILES_FOUND="YES"
      elif [ -f $KERNEL_SOURCE/$file ] ; then
         IMPORTANT_FILES_FOUND="YES"
      fi
   done 

   IMPORTANT_DIRS_FOUND="NO"
   for dir in $IMPORTANT_PACKAGE_DIRS ; do
      if [ -f $KERNEL_BUILD/$dir ] ; then
         IMPORTANT_DIRS_FOUND="YES"
      elif [ -f $KERNEL_SOURCE/$dir ] ; then
         IMPORTANT_DIRS_FOUND="YES"
      fi
   done 
   
   if [ "$IMPORTANT_FILES_FOUND" != "YES" -o "$IMPORTANT_DIRS_FOUND" != "YES" ] ; then
      print_missing_kernel_devel
   fi
}

copy_firmware( )
{
   FW=`find ./firmware -iname "dgc*fw*.bin"`
   if [ "x" = "x$FW" ] ; then
     echo "Failed to find any firmware images: $FW"
     echo "Failed to find any firmware images: $FW" >> install.log
     print_help
     exit
   fi
   echo -n "Copying Vision firmware..."
   echo -n "Copying Vision firmware..." >> install.log
   cp $FW /lib/firmware/ >> install.log 2>&1
   RET=$?
   if [ $RET -ne 0 ] ; then
     echo "FAILED"
     echo "FAILED" >> install.log
     echo "Failed to copy $FW into /lib/firmware : $RET"
     echo "Failed to copy $FW into /lib/firmware : $RET" >> install.log
     print_help
     exit
   fi
   echo "DONE"
   echo "DONE" >> install.log
}

build_module( )
{
   echo -n "Building module for kernel: $KVERSION.$KPATCHLEVEL.$KSUBLEVEL..."
   echo -n "Building module for kernel: $KVERSION.$KPATCHLEVEL.$KSUBLEVEL..." >> install.log
   start_progress build
   make RELEASE_BUILD=YES $1 modules >> install.log 2>&1
   RET=$?
   stop_progress build
   if [ $RET -ne 0 ] ; then
     echo "FAILED"
     echo "FAILED" >> install.log
     echo "Failed to build module: $RET"
     echo "Failed to build module: $RET" >> install.log
     print_help
     exit
   fi
   echo "DONE"
   echo "DONE" >> install.log
}

install_module( )
{
   if [ -e /etc/init.d/DGC133 ] ; then
      echo "Stopping User Mode service..."
      echo "Stopping User Mode service..." >> install.log
      /etc/init.d/DGC133 stop >> install.log
   fi
   
   echo -n "Installing module..."
   echo -n "Installing module..." >> install.log
   make modules_install >> install.log 2>&1
   RET=$?
   if [ $RET -ne 0 ] ; then
     echo "FAILED"
     echo "FAILED" >> install.log
     echo "Failed to install module: $RET"
     echo "Failed to install module: $RET" >> install.log
     print_help
     exit
   fi
   echo "DONE"
   echo "DONE" >> install.log
}

build_module_and_install( )
{
   build_module
   install_module
}

build_user_module_and_install( )
{
   build_module "RGB133_USER_MODE=YES"
   install_module
}

load_module( )
{
   echo -n "Loading module..."
   echo -n "Loading module..." >> install.log
   start_progress load
   make DEPMOD=$1 load >> install.log 2>&1
   RET=$?
   stop_progress load
   if [ $RET -ne 0 ] ; then
     echo "FAILED"
     echo "FAILED" >> install.log
     echo "Failed to load module: $RET"
     echo "Failed to load module: $RET" >> install.log
     print_help
     exit
   fi
   echo "DONE"
   echo "DONE" >> install.log
}

start_user_mode_service( )
{
   TRIES=0
   echo -n "Start User Mode Service..."
   echo -n "Start User Mode Service..." >> install.log

   start_progress ctrl
   while [ ! -c /dev/video63 -a $TRIES -ne 10 ] ; do
      sleep 1
      TRIES=$(($TRIES + 1))
   done
   stop_progress ctrl

   if [ ! -c /dev/video63 ] ; then
      echo "FAILED"
      echo "FAILED" >> install.log
      echo "Vision control device failed to initialise..."
      echo "Vision control device failed to initialise..." >> install.log
   else
      if [ -e /etc/init.d/DGC133 ] ; then
         start_progress ums
         /etc/init.d/DGC133 start >> install.log

         # Give the app init time
         sleep 20
         stop_progress ums
         echo "DONE"
         echo "DONE" >> install.log
      else
         echo "FAILED"
         echo "FAILED" >> install.log
         echo "User Mode service script missing."
         echo "User Mode service script missing." >> install.log
      fi
   fi
}

install_user_mode_service ( )
{
   echo -n "Install User Mode Service..."
   echo -n "Install User Mode Service..." >> install.log

   UPDATERCD=`which update-rc.d > /dev/null 2>&1`

   # Copy exe to usr local
   mkdir -p /usr/local/DGC133
   cp ./bin/DGC133 /usr/local/DGC133/DGC133

   # Copy startup script
   cp -vf ./scripts/DGC133 /etc/init.d/ >> install.log
   chmod uga+x /etc/init.d/DGC133

   MANUAL="NO"
   if [ -z "$UPDATERCD" ] ; then
      MANUAL="YES"
   elif [ ! -e $UPDATERCD ] ; then
      MANUAL="YES"
   fi

   if [ "$MANUAL" = "YES" ] ; then
      echo -n "Manually configuring system startup..."
      echo -n "Manually configuring system startup..." >> install.log

      # Remove existing startup entries
      find /etc -iname "S98DGC133" -exec rm -vf {} \; >> install.log
      find /etc -iname "K98DGC133" -exec rm -vf {} \; >> install.log

      # Add new startup entries
      cd /etc/rc0.d > /dev/null 2>&1
      ln -sf ../init.d/DGC133 K98DGC133 > /dev/null 2>&1
      cd - > /dev/null 2>&1
      cd /etc/rc1.d > /dev/null 2>&1
      ln -sf ../init.d/DGC133 K98DGC133 > /dev/null 2>&1
      cd - > /dev/null 2>&1
      cd /etc/rc2.d > /dev/null 2>&1
      ln -sf ../init.d/DGC133 S98DGC133 > /dev/null 2>&1
      cd - > /dev/null 2>&1
      cd /etc/rc2.d > /dev/null 2>&1
      ln -sf ../init.d/DGC133 K98DGC133 > /dev/null 2>&1
      cd - > /dev/null 2>&1
      cd /etc/rc3.d > /dev/null 2>&1
      ln -sf ../init.d/DGC133 S98DGC133 > /dev/null 2>&1
      cd - > /dev/null 2>&1
      cd /etc/rc4.d > /dev/null 2>&1
      ln -sf ../init.d/DGC133 S98DGC133 > /dev/null 2>&1
      cd - > /dev/null 2>&1
      cd /etc/rc5.d > /dev/null 2>&1
      ln -sf ../init.d/DGC133 S98DGC133 > /dev/null 2>&1
      cd - > /dev/null 2>&1
      cd /etc/rc6.d > /dev/null 2>&1
      ln -sf ../init.d/DGC133 K98DGC133 > /dev/null 2>&1
      cd - > /dev/null 2>&1

   else
      if [ -e /etc/init.d/DGC133 ] ; then
         # Remove existing startup entries
         $UPDATERCD -f DGC133 remove >> install.log
      fi

      # Add new startup entries
      $UPDATERCD DGC133 start 98 2 3 4 5 . stop 98 0 1 6 . >> install.log

   fi

   echo "DONE"
   echo "DONE" >> install.log
}

install_Vision ( )
{
   echo -n "Checking presence of Qt..."
   echo -n "Checking presence of Qt..." >> install.log
   ./bin/appconfigQt 2>> install.log
   RET=$?
   if [ $RET -ne 0 ] ; then
      echo "NOT FOUND"
      echo "NOT FOUND" >> install.log
      print_qt_help
      return
   fi

   echo "FOUND"
   echo "FOUND" >> install.log

   # Copy exe's to usr local

   echo -n "Installing Vision into /usr/local/Vision..."
   echo -n "Installing Vision into /usr/local/Vision..." >> install.log
   mkdir -p /usr/local/Vision > /dev/null 2>&1
   cp -f ./bin/Vision /usr/local/Vision/Vision > /dev/null 2>&1

   echo -n "Installing ForceDetect into /usr/local/Vision..."
   echo -n "Installing ForceDetect into /usr/local/Vision..." >> install.log
   cp -f ./bin/ForceDetect /usr/local/Vision/ForceDetect > /dev/null 2>&1

   echo -n "Installing Edid into /usr/local/Vision..."
   echo -n "Installing Edid into /usr/local/Vision..." >> install.log
   cp -f ./bin/Edid /usr/local/Vision/Edid > /dev/null 2>&1

   echo -n "Installing RGBHelper into /usr/local/Vision..."
   echo -n "Installing RGBHelper into /usr/local/Vision..." >> install.log
   cp -f ./bin/RGBHelper /usr/local/Vision/RGBHelper > /dev/null 2>&1

   echo "DONE"
   echo "DONE" >> install.log
}

start_links( )
{
   TRIES=0
   echo -n "Start Links Service..."
   echo -n "Start Links Service..." >> install.log

   start_progress ctrl
   while [ ! -c /dev/video63 -a $TRIES -ne 10 ] ; do
      sleep 1
      TRIES=$(($TRIES + 1))
   done
   stop_progress ctrl

   if [ ! -c /dev/video63 ] ; then
      echo "FAILED"
      echo "FAILED" >> install.log
      echo "Vision control device failed to initialise..."
      echo "Vision control device failed to initialise..." >> install.log
   else
      if [ -e /etc/init.d/RGB133Links ] ; then
         start_progress links
         
         # Ubuntu 15.04 uses systemd by default but because it still can use System V, we choose to do it the old way.
         # The way it is done is systemd on Ubuntu 15.04 creates RGB133Links.service config files
         # (does not use RGB133Links.service which we provide in <install_dir>/configurations)
         # In such case, to avoid warning messages that the config files have changed on driver reinstall,
         # issue this command to reload systemd configuration before it will actually try to start the service
         if [ -d /etc/systemd/ ] ; then
            #We have /etc/systemd/ but do we have the command to control the service manager?
            type systemctl >/dev/null 2>&1
            if [ $? -eq 0 ]; then
               systemctl daemon-reload >> install.log 2>&1
            fi
         fi
         
         /etc/init.d/RGB133Links start >> install.log

         # Give the app init time
         sleep 1
         stop_progress links
         echo "DONE"
         echo "DONE" >> install.log
      elif [ -e /etc/systemd/system/RGB133Links.service ] ; then 
         start_progress links 
         # enable RGB133Links.service to start at boot time
         echo -en "\n" >> install.log
         systemctl enable RGB133Links.service >> install.log 2>&1 
         systemctl start RGB133Links.service >> install.log 2>&1
         # Give the app init time
         sleep 1
         stop_progress links
         echo "DONE"
         echo "DONE" >> install.log
      else
         echo "FAILED"
         echo "FAILED" >> install.log
         echo "Links service script missing."
         echo "Links service script missing." >> install.log
      fi
   fi
}

install_links ( )
{
   DIR_OUT="/etc/init.d/"
   echo -n "Installing RGB133Links into /usr/local/DGC133..."
   echo -n "Installing RGB133Links into /usr/local/DGC133..." >> install.log

   UPDATERCD=`which update-rc.d > /dev/null 2>&1`

   # Copy exe to usr local
   mkdir -p /usr/local/DGC133 > /dev/null 2>&1
   cp -f ./bin/RGB133Links /usr/local/DGC133/RGB133Links > /dev/null 2>&1

   # Copy startup script; do it only if dir /etc/init.d/ exists.
   if [ -d "$DIR_OUT" ]; then 
      cp -vf ./scripts/RGB133Links /etc/init.d/ >> install.log
      chmod uga+x /etc/init.d/RGB133Links

      MANUAL="NO"
      if [ -z "$UPDATERCD" ] ; then
         MANUAL="YES"
      elif [ ! -e $UPDATERCD ] ; then
         MANUAL="YES"
      fi

      if [ "$MANUAL" = "YES" ] ; then
         echo -n "Manually configuring system startup..."
         echo -n "Manually configuring system startup..." >> install.log

         # Remove existing startup entries
         find /etc -iname "S99RGB133Links" -exec rm -vf {} \; >> install.log

         # Add new startup entries
         cd /etc/rc2.d > /dev/null 2>&1
         ln -sf ../init.d/RGB133Links S99RGB133Links > /dev/null 2>&1
         cd - > /dev/null 2>&1
         cd /etc/rc3.d > /dev/null 2>&1
         ln -sf ../init.d/RGB133Links S99RGB133Links > /dev/null 2>&1
         cd - > /dev/null 2>&1
         cd /etc/rc4.d > /dev/null 2>&1
         ln -sf ../init.d/RGB133Links S99RGB133Links > /dev/null 2>&1
         cd - > /dev/null 2>&1
         cd /etc/rc5.d > /dev/null 2>&1
         ln -sf ../init.d/RGB133Links S99RGB133Links > /dev/null 2>&1
         cd - > /dev/null 2>&1

      else
         if [ -e /etc/init.d/DGC133 ] ; then
            # Remove existing startup entries
            $UPDATERCD -f RGB133Links remove >> install.log
         fi

         # Add new startup entries
         $UPDATERCD RGB133Links start 98 2 3 4 5 . >> install.log

      fi

   else
      # Using systemd service management
      DIR_SYSTEM="/etc/systemd/system"
      USR_LOCAL_SYSTEM_DIR="/usr/local/lib/systemd/system"

      if [ -d "$DIR_SYSTEM" ]; then
         # Copy .service file to usr local lib
         mkdir -p /usr/local/lib/systemd/system > /dev/null 2>&1
           
         # Remove existing startup .service file link
         find $DIR_SYSTEM -iname "RGB133Links.service" -exec rm -vf {} \; >> install.log

         # Copy startup .service file and create symbolic link to it
         cp -vf ./configurations/RGB133Links.service $USR_LOCAL_SYSTEM_DIR/ >> install.log
         ln -sf $USR_LOCAL_SYSTEM_DIR/RGB133Links.service $DIR_SYSTEM/RGB133Links.service > /dev/null 2>&1   
      fi
   fi 

   echo "DONE"
   echo "DONE" >> install.log
}

install_conf ( )
{
   if [ -d /etc/modprobe.d ] ; then
     if [ ! -e /etc/modprobe.d/rgb133.sample.conf ] ; then
       INSTALL_RGB133_CONF="YES"
     else
       CMP=`cmp ./scripts/rgb133.sample.conf /etc/modprobe.d/rgb133.sample.conf`
       RET=$?
       if [ $RET -ne 0 ] ; then
         if [ "$SILENT" != "YES" ] ; then
           echo -n "Overwrite existing rgb133.sample.conf file [y/N]: "
           echo -n "Overwrite existing rgb133.sample.conf file [y/N]: " >> install.log
           read ow
           if [ "$ow" = "y" -o "$ow" = "Y" ] ; then
             INSTALL_RGB133_CONF="YES"
           fi
         fi
       fi
     fi
     
     if [ "$INSTALL_RGB133_CONF" = "YES" ] ; then
       echo -n "Installing rgb133.sample.conf into /etc/modprobe.d/..."
       echo -n "Installing rgb133.sample.conf into /etc/modprobe.d/..." >> install.log

       # Copy file
       cp -vf ./scripts/rgb133.sample.conf /etc/modprobe.d/ >> install.log

       echo "DONE"
       echo "DONE" >> install.log
     else
       echo -n "Skipping installation of rgb133.sample.conf [file exists]..."
       echo -n "Skipping installation of rgb133.sample.conf [file exists]..." >> install.log
     fi
   else
     echo -n "Skipping installation of rgb133.sample.conf..."
     echo -n "Skipping installation of rgb133.sample.conf..." >> install.log
   fi
}

upgrade_flash ( )
{
   DEVICES="NO"
   MODIFY="NO"
   FLASHIMG=`find ./firmware -name "flashimg*"`
   if [ -z "$FLASHIMG" ] ; then
      echo "Failed to find suitable flash image.  Firmware may not be usable..."
      echo "Failed to find suitable flash image.  Firmware may not be usable..." >> install.log
      echo "ls -al ./firmware" >> install.log
      ls -al ./firmware >> install.log
      print_help
      return
   fi
   
   echo -n "Checking for flashable devices, please wait..."
   echo -n "Checking for flashable devices, please wait..." >> install.log
   start_progress list

   ./bin/Flash133 -l >> install.log
   RET=$?
   stop_progress list
   if [ $RET -gt 0 ] ; then
      echo "DONE"
      echo "Flashable devices found..."
      echo "Flashable devices found..." >> install.log
      DEVICES="YES"
   elif [ $RET -eq 0 ] ; then
      echo "DONE"
      echo "No flashable devices found..."
      echo "No flashable devices found..." >> install.log
      DEVICES="NO"
   else
      echo "FAILED($RET)"
      DEVICES="NO"
   fi

   if [ "$DEVICES" = "YES" ] ; then
      echo -n "Checking current flash version(s), please wait..."
      echo -n "Checking current flash version(s), please wait..." >> install.log
      start_progress comp

      ################### TEMP ###################
      # Delay until IRQ handshake has finished.  #
      sleep 5
      ############################################

      ./bin/Flash133 -c -f $FLASHIMG >> install.log
      RET=$?
      stop_progress comp
      if [ $RET -eq 254 ] ; then
         echo "DONE"
         echo -n "Flash requires downgrading, please wait..."
         echo -n "Flash requires downgrading, please wait..." >> install.log
         MODIFY="YES"
      elif [ $RET -eq 255 ] ; then
         echo "DONE"
         echo -n "Flash requires upgrading, please wait..."
         echo -n "Flash requires upgrading, please wait..." >> install.log
         MODIFY="YES"
      elif [ $RET -eq 0 ] ; then
         echo "DONE"
         echo "Flash is up to date..."
         echo "Flash is up to date..." >> install.log
      else
         echo "FAILED($RET)"
      fi

      if [ "$MODIFY" = "YES" ] ; then
         start_progress write
         ./bin/Flash133 -w -f $FLASHIMG >> install.log
         stop_progress write
         echo "DONE"
         echo "DONE" >> install.log
         print_shutdown
      fi
   fi
}

cleanup_arch_files ( )
{
   FILES=`find ./bin -name "*.$NOT_ARCH_BITS*"`
   rm -vf $FILES >> install.log
   if [ -e ./bin/rgb133.$RT$ARCH_BITS.a ] ; then
      mv ./bin/rgb133.$RT$ARCH_BITS.a ./bin/rgb133.a
      if [ -e ./bin/rgb133.$ARCH_BITS.a ]  ; then
         rm -vf ./bin/rgb133.$ARCH_BITS.a >> install.log
      elif [ -e ./bin/rgb133.RT.$ARCH_BITS.a ] ; then
         rm -vf ./bin/rgb133.RT.$ARCH_BITS.a >> install.log
      fi
   fi
   if [ -e ./bin/dgc133sys.$RT$ARCH_BITS.a ] ; then
      mv ./bin/dgc133sys.$RT$ARCH_BITS.a ./bin/dgc133sys.a
      if [ -e ./bin/dgc133sys.$ARCH_BITS.a ]  ; then
         rm -vf ./bin/dgc133sys.$ARCH_BITS.a >> install.log
      elif [ -e ./bin/dgc133sys.RT.$ARCH_BITS.a ] ; then
         rm -vf ./bin/dgc133sys.RT.$ARCH_BITS.a >> install.log
      fi
   fi
   if [ -e ./bin/DGC133.$ARCH_BITS.static ] ; then
      mv ./bin/DGC133.$ARCH_BITS.static ./bin/DGC133
   fi
   if [ -e ./bin/Flash133.$ARCH_BITS ] ; then
      mv ./bin/Flash133.$ARCH_BITS ./bin/Flash133
   fi
   if [ -e ./bin/appconfigQt.$ARCH_BITS ] ; then
      mv ./bin/appconfigQt.$ARCH_BITS ./bin/appconfigQt
   fi
   if [ -e ./bin/Vision.$ARCH_BITS ] ; then
      mv ./bin/Vision.$ARCH_BITS ./bin/Vision
   fi
   if [ -e ./bin/RGB133Links.$ARCH_BITS ] ; then
      mv ./bin/RGB133Links.$ARCH_BITS ./bin/RGB133Links
   fi
   if [ -e ./bin/ForceDetect.$ARCH_BITS ] ; then
      mv ./bin/ForceDetect.$ARCH_BITS ./bin/ForceDetect
   fi
   if [ -e ./bin/Edid.$ARCH_BITS ] ; then
      mv ./bin/Edid.$ARCH_BITS ./bin/Edid
   fi
   if [ -e ./bin/RGBHelper.$ARCH_BITS ] ; then
      mv ./bin/RGBHelper.$ARCH_BITS ./bin/RGBHelper
   fi
   if [ -e ./bin/RGB133Debug.$ARCH_BITS ] ; then
      mv ./bin/RGB133Debug.$ARCH_BITS ./bin/RGB133Debug
   fi
}

## Script start
if [ "$VERSION" = "YES" ] ; then
   print_long_version
   exit
fi

echo "Vision Linux Driver Installer..."
echo "Vision Linux Driver Installer..." > install.log

echo -n "Checking for previous installation..."
echo -n "Checking for previous installation..." >> install.log

KERNEL_STR=`uname -r`
RGB133_MODULE=`find /lib/modules/$KERNEL_STR -iname "rgb133.ko*"`
Vision_DIR=`find /usr/local/ -iname "Vision"`

if [ -n "$RGB133_MODULE" -o -n "$Vision_DIR" ] ; then
   echo "FOUND"
   echo "FOUND" >> install.log

   if [ "$SILENT" != "YES" ] ; then
      echo -n "Do you wish to remove the previous installation (Y/N) [N]: "
      read cont

      if [ "$cont" != "YES" -a "$cont" != "yes" -a \
         "$cont" != "Y" -a "$cont" != "y" ] ; then
         echo
         echo "Cancelling installation, previous installation unmodified."
         exit
      fi
   fi

   # Remove previous installation
   ./scripts/uninstall.sh -s
else
   echo "NONE"
   echo "NONE" >> install.log
fi

print_header

if [ `id -u` -ne 0 ] ; then
   echo "This installer needs to be run as the superuser"
   echo "This installer needs to be run as the superuser" >> install.log
   exit
fi

LIC_TEXT=`find ./docs -iname "LICENCE"`
USE_LICENCE="NO"
if [ -n "$LIC_TEXT" ] ; then
   USE_LICENCE="YES"
   print_license
fi

echo "Beginning install, please wait..."
echo "Beginning install, please wait..." >> install.log

# Check for important files
IMPORTANT="dgc133fw.bin"
echo -n "Check for important files..."
echo -n "Check for important files..." >> install.log
for file in $IMPORTANT ; do
  FOUND=`find . -iname "$file"`
  if [ -z "$FOUND" ] ; then
     echo "FAILED"
     echo "FAILED" >> install.log
     echo "$0: $file is missing, please re-unpack the installation"
     echo "$0: $file is missing, please re-unpack the installation" >> install.log
     echo "    package and try again."
     echo "    package and try again." >> install.log
     print_help
     exit
  fi
done

# Look for depmod, start with the most common locations...
SEARCH_DIRS="/sbin/ /bin/ /usr/bin/ /usr/sbin/"
DEPMOD=""
for dir in $SEARCH_DIRS ; do
   DEPMOD=`find $dir -name "depmod"`
   if [ -n "$DEPMOD" ] ; then
      # we found depmod; do not search further
      break
   fi
done

if [ -z "$DEPMOD" ] ; then
   echo "FAILED"
   echo "FAILED" >> install.log
   echo "$0: Failed to find command: depmod"
   echo "$0: Failed to find command: depmod"  >> install.log
   print_help
fi

echo "DONE"
echo "DONE" >> install.log

# Cleanup unnecessary files
echo -n "Performing pre-build cleanup..."
echo -n "Performing pre-build cleanup..." >> install.log
cleanup_arch_files
echo "DONE"
echo "DONE" >> install.log

# Build
MAKEOPTS=
if [ $KVERSION -gt 1 ] ; then
   if [ $KVERSION -eq 2 ] ; then
      if [ $KPATCHLEVEL -ne 6 ] ; then
         echo "Unsupported kernel patch level: $KERNELVERSION"
         echo "Unsupported kernel patch level: $KERNELVERSION" >> install.log
         print_help
         exit
      fi
   fi

   copy_firmware
   if [ "$USER_MODE" = "YES" ] ; then
      build_user_module_and_install
   else
      build_module_and_install
   fi
   load_module $DEPMOD
   if [ "$USER_MODE" = "YES" ] ; then
      install_user_mode_service
      start_user_mode_service
   fi
   install_Vision
   install_links
   install_conf
   start_links
   upgrade_flash
else
   echo "Unsupported kernel version: $KERNELVERSION"
   echo "Unsupported kernel version: $KERNELVERSION" >> install.log
   print_help
   exit
fi

echo
echo >> install.log
echo "====================================================================="
echo "=====================================================================" >> install.log
echo "== Vision Linux Driver Installer Complete!"
echo "== Vision Linux Driver Installer Complete!" >> install.log
echo "====================================================================="
echo "=====================================================================" >> install.log

